# -*- coding: utf-8 -*-

DATABASE_MIRRORS = [
    "https://raw.githubusercontent.com/pyupio/safety-db/master/data/",
    # S3 mirrors
    "https://s3.eu-central-1.amazonaws.com/safety-db-mirror-1/data/",
    "https://s3-us-west-2.amazonaws.com/safety-db-mirror-2/data/",
    "https://s3.amazonaws.com/safety-db-mirror-3/data/"
]
REQUEST_TIMEOUT = 5


